/*
    data.c - Part of psyconv, a PSION 5 data formats converter
    Copyright (c) 1999  Frodo Looijaard <frodol@dds.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include "data.h"
#include "list.h"
#include "general.h"

cm_length_t to_cm_length(u32 len)
{
  return (2.54/1440.0) * ((s32) len);
}

point_size_t to_point_size(u32 len)
{
  return len / 20.0;
}

bool_t to_bool(u8 val)
{
  return val?bool_true:bool_false;
}

character_layout_status basic_character_layout_status(void)
{
  struct character_layout_status cl = 
  {
    { 0, 0, 0},    /* color               */
    { 0, 0, 0},    /* back_color          */
    10.0,          /* font_size           */
    bool_false,    /* italic              */
    bool_false,    /* bold                */
    normalscript,  /* super_sub           */
    bool_false,    /* underline           */
    bool_false,    /* crossed_out         */
    -1,            /* font                */
  };

  return clone_character_layout_status(&cl);
}


paragraph_layout_status basic_paragraph_layout_status(void)
{
  struct paragraph_layout_status pl =
  {
    0.0,           /* indent_left          */
    0.0,           /* indent_right         */
    0.0,           /* indent_first         */
    justify_left,  /* justify             */
    0.0,           /* interline           */
    bool_false,    /* interline_exact     */
    0.0,           /* top_space           */
    0.0,           /* bottom_space        */
    bool_false,    /* on_one_page         */
    bool_false,    /* together_with       */
    bool_false,    /* on_next_page        */
    bool_false,    /* no_widow_protection */
    0.0,           /* left_margin         */
    {              /* bullet              */
      bool_false,    /* on                  */
      bool_true,     /* indent              */
      10.0,          /* font_size           */
      -1,            /* font                */
      0x95,          /* character           */
      { 0,0,0}       /*  color              */
    },        
    {              /* left_border         */
      border_none,   /* kind                */
      1,             /* thickness           */
      { 0,0,0 }      /* color               */
    },
    {              /* right_border        */
      border_none,   /* kind              */
      1,             /* thickness         */
      { 0,0,0 }      /* color             */
    },
    {              /* top_border          */
      border_none,   /* kind                */
      1,             /* thickness           */
      { 0,0,0 }      /* color               */
    },
    {              /* bottom_border       */
      border_none,   /* kind                */
      1,             /* thickness           */
      { 0,0,0 }      /* color               */
    },
    {              /* tabs            */
      {              /* normal              */
        0.64,          /* location            */
        tab_left,      /* kind                */
      },
      NULL           /* extras */
    }
  };
  paragraph_layout_status res;
  
  pl.tabs.extras = list_new(sizeof(tab_t));
  res = clone_paragraph_layout_status(&pl);
  list_free(pl.tabs.extras);
  return res;
}
  
  
character_layout_status clone_character_layout_status
                                                  (character_layout_status ls)
{
  character_layout_status result;

  result = malloc(sizeof(*result));
  *result = *ls;
  return result;
}

paragraph_layout_status clone_paragraph_layout_status
                                                 (paragraph_layout_status ls)
{
  paragraph_layout_status result;

  result = malloc(sizeof(*result));
  *result = *ls;
  result->tabs.extras = list_clone(result->tabs.extras);
  return result;
}

style get_style (styles_section ss, int nr)
{
  if (nr == 0)
    return ss->normal;
  else
    return list_get(ss->styles,0xff - nr);
}

font_desc get_font(font_list fl, int nr)
{
  int i;
  font_desc res;
  for (i = 0; i < list_length(fl); i ++)  {
    res = list_get(fl,i);
    if (res->id == nr)
      return res;
  }
  return NULL;
}

