/*
    list.c - Part of psyconv, a PSION 5 data formats converter
    Copyright (c) 1999  Frodo Looijaard <frodol@dds.nl>       

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include "list.h"

static void list_resize(list l,unsigned int nr);

struct list {
  int cur_len;
  int max_len;
  int el_size;
  void *els;
};

list list_new(int element_size)
{
  list l;
  l = malloc(sizeof(*l));
  l->cur_len = 0;
  l->max_len = 0;
  l->el_size=element_size;
  l->els = NULL; 
  return l;
} 

void list_free(list l)
{
  if (l->max_len)
    free(l->els);
  l->max_len = 0;
  l->cur_len = 0;
  l->els = NULL;
}

void list_free_el(list l, void free_el(void *el))
{
  list_foreach_el(l,free_el);
  list_free(l);
}

int list_length(const list l)
{
  return l->cur_len;
}

int list_is_empty(const list l)
{
  return l->cur_len == 0;
}

void *list_get(const list l, unsigned int indx)
{
  if (indx >= l->cur_len)
    return NULL;
  else
    return ((char *) (l->els)) + indx * l->el_size;
}

void list_add(list l, void *el)
{
  list_resize(l,l->cur_len + 1);
  memcpy(((char *) (l->els)) + l->cur_len * l->el_size, el, l->el_size);
  l->cur_len ++;
}

void list_foreach_el(list l, void action(void *el))
{
  int i;
  for (i = 0; i < l->cur_len; i ++)
    action(list_get(l,i));
}

list list_clone(const list l)
{
  list l2;
  int i;
  l2 = list_new(l->el_size);
  for (i = 0; i < l->cur_len; i ++)
    list_add(l2,list_get(l,i));
  return l2;
  
}

list list_clone_el(const list l,void clone_el(void *el))
{
  list l2 = list_clone(l);
  list_foreach_el(l2,clone_el);
  return l2;
}

size_t list_fread(list l,size_t size, FILE *f)
{
  size_t res;
  list_resize(l,l->cur_len + size);
  res = fread(((char *) (l->els)) + l->cur_len * l->el_size,l->el_size,size,f);
  l->cur_len += res;
  return res;
}

void list_resize(list l,unsigned int nr)
{
  if (nr > l->max_len) {
    l->max_len = 1.1 * nr;
    l->max_len += 16 - l->max_len % 16;
    l->els = realloc(l->els,l->max_len * l->el_size);
  }
}

