/*
    tools.c - Part of psyconv, a PSION 5 data formats converter
    Copyright (c) 1999  Frodo Looijaard <frodol@dds.nl>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdarg.h>
#include <stdio.h>
#include "tools.h"
#include "main.h"

void fatal(const char *format,...)
{
  va_list ap;
  va_start(ap,format);
  fputs("Fatal error: ",stderr);
  vfprintf(stderr,format,ap);
  fputs("\n",stderr);
  va_end(ap);
  exit(1);
}

void progress(const char *format,...)
{
  va_list ap;
  va_start(ap,format);
  if (verbosity >= VERB_PROGRESS) {
    fputs("-->",stderr);
    vfprintf(stderr,format,ap);
    fputs("\n",stderr);
  }
  va_end(ap);
}

void warn(const char *format,...) 
{
  va_list ap;
  va_start(ap,format);
  if (verbosity >= VERB_WARN) {
    fputs("WARNING: ",stderr);
    vfprintf(stderr,format,ap);
    fputs("\n",stderr);
  }
  va_end(ap);
}

void debug(const char *format,...)
{
  va_list ap;
  va_start(ap,format);
  if (verbosity >= VERB_DEBUG) {
    fputs("...",stderr);
    vfprintf(stderr,format,ap);
    fputs("\n",stderr);
  }
  va_end(ap);
}


